@@ -24,6 +24,7 @@
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "absl/strings/substitute.h"
+#include "google/protobuf/compiler/code_generator_lite.h"
 #include "google/protobuf/compiler/java/context.h"
 #include "google/protobuf/compiler/java/doc_comment.h"
 #include "google/protobuf/compiler/java/field_common.h"
@@ -36,6 +37,7 @@
 #include "google/protobuf/compiler/java/full/message_builder.h"
 #include "google/protobuf/compiler/java/message_serialization.h"
 #include "google/protobuf/compiler/java/name_resolver.h"
+#include "google/protobuf/compiler/java/names.h"
 #include "google/protobuf/descriptor.h"
 #include "google/protobuf/descriptor.pb.h"
 #include "google/protobuf/io/coded_stream.h"
@@ -103,12 +105,12 @@ void ImmutableMessageGenerator::GenerateStaticVariables(
   if (descriptor_->containing_type() != nullptr) {
     vars["parent"] = UniqueFileScopeIdentifier(descriptor_->containing_type());
   }
-  if (MultipleJavaFiles(descriptor_->file(), /* immutable = */ true)) {
+  if (NestedInFileClass(*descriptor_)) {
+    vars["private"] = "private ";
+  } else {
     // We can only make these package-private since the classes that use them
     // are in separate files.
     vars["private"] = "";
-  } else {
-    vars["private"] = "private ";
   }
   if (*bytecode_estimate <= kMaxStaticSize) {
     vars["final"] = "final ";
@@ -152,13 +154,12 @@ int ImmutableMessageGenerator::GenerateStaticVariableInitializers(
   if (descriptor_->containing_type() == nullptr) {
     printer->Print(vars,
                    "internal_$identifier$_descriptor =\n"
-                   "  getDescriptor().getMessageTypes().get($index$);\n");
+                   "  getDescriptor().getMessageType($index$);\n");
     bytecode_estimate += 30;
   } else {
-    printer->Print(
-        vars,
-        "internal_$identifier$_descriptor =\n"
-        "  internal_$parent$_descriptor.getNestedTypes().get($index$);\n");
+    printer->Print(vars,
+                   "internal_$identifier$_descriptor =\n"
+                   "  internal_$parent$_descriptor.getNestedType($index$);\n");
     bytecode_estimate += 30;
   }
 
@@ -179,12 +180,12 @@ void ImmutableMessageGenerator::GenerateFieldAccessorTable(
     io::Printer* printer, int* bytecode_estimate) {
   absl::flat_hash_map<absl::string_view, std::string> vars;
   vars["identifier"] = UniqueFileScopeIdentifier(descriptor_);
-  if (MultipleJavaFiles(descriptor_->file(), /* immutable = */ true)) {
+  if (NestedInFileClass(*descriptor_)) {
+    vars["private"] = "private ";
+  } else {
     // We can only make these package-private since the classes that use them
     // are in separate files.
     vars["private"] = "";
-  } else {
-    vars["private"] = "private ";
   }
   if (*bytecode_estimate <= kMaxStaticSize) {
     vars["final"] = "final ";
@@ -241,7 +242,7 @@ int ImmutableMessageGenerator::GenerateFieldAccessorTableInitializer(
 void ImmutableMessageGenerator::GenerateInterface(io::Printer* printer) {
   MaybePrintGeneratedAnnotation(context_, printer, descriptor_,
                                 /* immutable = */ true, "OrBuilder");
-  if (!context_->options().opensource_runtime) {
+  if (!google::protobuf::internal::IsOss()) {
     printer->Print("@com.google.protobuf.Internal.ProtoNonnullApi\n");
   }
   if (descriptor_->extension_range_count() > 0) {
@@ -302,7 +303,7 @@ void ImmutableMessageGenerator::Generate(io::Printer* printer) {
   WriteMessageDocComment(printer, descriptor_, context_->options());
   MaybePrintGeneratedAnnotation(context_, printer, descriptor_,
                                 /* immutable = */ true);
-  if (!context_->options().opensource_runtime) {
+  if (!google::protobuf::internal::IsOss()) {
     printer->Print("@com.google.protobuf.Internal.ProtoNonnullApi\n");
   }
 
@@ -339,7 +340,7 @@ void ImmutableMessageGenerator::Generate(io::Printer* printer) {
 
   printer->Print("static {\n");
   printer->Indent();
-  PrintGencodeVersionValidator(printer, context_->options().opensource_runtime,
+  PrintGencodeVersionValidator(printer, google::protobuf::internal::IsOss(),
                                descriptor_->name());
   printer->Outdent();
   printer->Print("}\n");
@@ -432,7 +433,7 @@ void ImmutableMessageGenerator::Generate(io::Printer* printer) {
                    "private $oneof_capitalized_name$Case(int value) {\n"
                    "  this.value = value;\n"
                    "}\n");
-    if (context_->options().opensource_runtime) {
+    if (google::protobuf::internal::IsOss()) {
       printer->Print(
           vars,
           "/**\n"
@@ -446,7 +447,7 @@ void ImmutableMessageGenerator::Generate(io::Printer* printer) {
           "}\n"
           "\n");
     }
-    if (!context_->options().opensource_runtime) {
+    if (!google::protobuf::internal::IsOss()) {
       printer->Print(
           "@com.google.protobuf.Internal.ProtoMethodMayReturnNull\n");
     }
@@ -733,7 +734,7 @@ void ImmutableMessageGenerator::GenerateParseFromMethods(io::Printer* printer) {
       "\n",
       "classname", name_resolver_->GetImmutableClassName(descriptor_),
       "parsedelimitedreturnannotation",
-      context_->options().opensource_runtime
+      google::protobuf::internal::IsOss()
           ? ""
           : "@com.google.protobuf.Internal.ProtoMethodMayReturnNull");
 }
@@ -877,53 +878,49 @@ void ImmutableMessageGenerator::GenerateIsInitialized(io::Printer* printer) {
     const FieldGeneratorInfo* info = context_->GetFieldGeneratorInfo(field);
     if (GetJavaType(field) == JAVATYPE_MESSAGE &&
         HasRequiredFields(field->message_type())) {
-      switch (field->label()) {
-        case FieldDescriptor::LABEL_REQUIRED:
+      if (field->is_required()) {
+        printer->Print(
+            "if (!get$name$().isInitialized()) {\n"
+            "  memoizedIsInitialized = 0;\n"
+            "  return false;\n"
+            "}\n",
+            "type",
+            name_resolver_->GetImmutableClassName(field->message_type()),
+            "name", info->capitalized_name);
+      } else if (field->is_repeated()) {
+        if (IsMapEntry(field->message_type())) {
           printer->Print(
-              "if (!get$name$().isInitialized()) {\n"
-              "  memoizedIsInitialized = 0;\n"
-              "  return false;\n"
-              "}\n",
-              "type",
-              name_resolver_->GetImmutableClassName(field->message_type()),
-              "name", info->capitalized_name);
-          break;
-        case FieldDescriptor::LABEL_OPTIONAL:
-          printer->Print(
-              "if (has$name$()) {\n"
-              "  if (!get$name$().isInitialized()) {\n"
+              "for ($type$ item : get$name$Map().values()) {\n"
+              "  if (!item.isInitialized()) {\n"
               "    memoizedIsInitialized = 0;\n"
               "    return false;\n"
               "  }\n"
               "}\n",
+              "type",
+              MapValueImmutableClassdName(field->message_type(),
+                                          name_resolver_),
               "name", info->capitalized_name);
-          break;
-        case FieldDescriptor::LABEL_REPEATED:
-          if (IsMapEntry(field->message_type())) {
-            printer->Print(
-                "for ($type$ item : get$name$Map().values()) {\n"
-                "  if (!item.isInitialized()) {\n"
-                "    memoizedIsInitialized = 0;\n"
-                "    return false;\n"
-                "  }\n"
-                "}\n",
-                "type",
-                MapValueImmutableClassdName(field->message_type(),
-                                            name_resolver_),
-                "name", info->capitalized_name);
-          } else {
-            printer->Print(
-                "for (int i = 0; i < get$name$Count(); i++) {\n"
-                "  if (!get$name$(i).isInitialized()) {\n"
-                "    memoizedIsInitialized = 0;\n"
-                "    return false;\n"
-                "  }\n"
-                "}\n",
-                "type",
-                name_resolver_->GetImmutableClassName(field->message_type()),
-                "name", info->capitalized_name);
-          }
-          break;
+        } else {
+          printer->Print(
+              "for (int i = 0; i < get$name$Count(); i++) {\n"
+              "  if (!get$name$(i).isInitialized()) {\n"
+              "    memoizedIsInitialized = 0;\n"
+              "    return false;\n"
+              "  }\n"
+              "}\n",
+              "type",
+              name_resolver_->GetImmutableClassName(field->message_type()),
+              "name", info->capitalized_name);
+        }
+      } else {
+        printer->Print(
+            "if (has$name$()) {\n"
+            "  if (!get$name$().isInitialized()) {\n"
+            "    memoizedIsInitialized = 0;\n"
+            "    return false;\n"
+            "  }\n"
+            "}\n",
+            "name", info->capitalized_name);
       }
     }
   }
@@ -953,7 +950,7 @@ void ImmutableMessageGenerator::GenerateEqualsAndHashCode(
   printer->Print(
       "@java.lang.Override\n"
       "public boolean equals(");
-  if (!context_->options().opensource_runtime) {
+  if (!google::protobuf::internal::IsOss()) {
     printer->Print(
         "@com.google.protobuf.Internal.ProtoMethodAcceptsNullParameter\n");
   }
@@ -1171,40 +1168,6 @@ void ImmutableMessageGenerator::GenerateInitializers(io::Printer* printer) {
   }
 }
 
-// ===================================================================
-void ImmutableMessageGenerator::GenerateMutableCopy(io::Printer* printer) {
-  printer->Print(
-      "protected com.google.protobuf.MutableMessage\n"
-      "    internalMutableDefault() {\n"
-      "  return MutableDefaultLoader.get();\n"
-      "}\n"
-      "\n"
-      "private static final class MutableDefaultLoader {\n"
-      "  private static final java.lang.Object defaultOrRuntimeException;\n"
-      "  static {\n"
-      "    java.lang.Object local;\n"
-      "    try {\n"
-      "      local = internalMutableDefault(\"$mutable_name$\");\n"
-      "    } catch (java.lang.RuntimeException e) {\n"
-      "      local = e;\n"
-      "    }\n"
-      "    defaultOrRuntimeException = local;\n"
-      "  }\n"
-      "\n"
-      "  private MutableDefaultLoader() {}\n"
-      "\n"
-      "  public static com.google.protobuf.MutableMessage get() {\n"
-      "    if (defaultOrRuntimeException\n"
-      "         instanceof java.lang.RuntimeException) {\n"
-      "      throw (java.lang.RuntimeException) defaultOrRuntimeException;\n"
-      "    }\n"
-      "    return\n"
-      "        (com.google.protobuf.MutableMessage) "
-      "defaultOrRuntimeException;\n"
-      "  }\n"
-      "}\n",
-      "mutable_name", name_resolver_->GetJavaMutableClassName(descriptor_));
-}
 
 void ImmutableMessageGenerator::GenerateAnyMethods(io::Printer* printer) {
   printer->Print(
